The Posit Workbench VSCode Extension allows seamless development in Posit Workbench with Visual Studio Code.

For more information using VSCode in Posit Workbench, take a look at the [VSCode chapter of the Posit Workbench User Guide](https://docs.posit.co/ide/server-pro/user/vs-code/getting-started/).
