@echo off
setlocal
set ROOT_DIR=%~dp0..\..\..\..
set VSROOT_DIR=%~dp0..\..
call "%ROOT_DIR%\node.exe" "%VSROOT_DIR%\out\server-cli.js" "code-server" "1.71.2" "74b1f979648cc44d385a2286793c226e611f59e7" "code-server.cmd" "--openExternal" %*
endlocal
