---
title: Untitled
format:
  <%= name %>-docx: default
author: <%= author %>
date: last-modified
---

## Introduction

*TODO* Create an example file that demonstrates the formatting and features of your format.

## More Information

You can learn more about controlling the appearance of Word output here: <https://quarto.org/docs/output-formats/ms-word.html>
